const path = require('path');
const webpack = require('webpack');
const Dotenv = require('dotenv-webpack');
const child_process = require('child_process');

module.exports = {
    mode: process.env.WEBPACK_MODE,
    entry: './src/index.js',
    output: {
      filename: 'main.js',
      path: path.resolve(__dirname, 'public/dist'),
      publicPath: '/dist/',
    },
    devServer: {
      hot: true,
      watchFiles: ['src/**/*.js'],
      client: {
          logging: 'verbose',
          overlay: true,
          progress: true,
          webSocketURL: 'ws://0.0.0.0:' + process.env.DEV_PORT + '/ws'
      },
      static: [
        {
            directory: path.join(__dirname, 'public'),
            watch: true
        },
        {
            directory: path.join(__dirname, process.env.HTTP_FOLDER ? process.env.HTTP_FOLDER : 'www'),
            watch: false,
            publicPath: '/data'
        },
      ],
      port: 9000
    },
    plugins: [
        new webpack.IgnorePlugin({
            /* cf https://gitlab.tetras-libre.fr/iiif/mirador-video-annotation/-/blob/annotation-on-video/webpack.config.js#L42 */
            resourceRegExp: /@blueprintjs\/(core|icons)/, // ignore optional UI framework dependencies
        }),
        {
            apply: (compiler) => {
                function spawn(cmd, args, cwd) {
                    let ret = child_process.spawnSync(cmd, args, {cwd: path.resolve(cwd)});
                    if (ret.output) {
                        console.log(ret.output.toString());
                    }
                    if (ret.status !== 0) {
                        throw Error('Error while running ""'+cmd+' '+args);
                    }
                }
                // TODO this hook runs again and again in watch mode, use something more restrictive
                compiler.hooks.compile.tap("compile workspaces", () => {
                    console.log('Compiling workspaces');
                    console.log('annotations plugin');
                    // TODO Use values from package.json
                    spawn("rm", ["-f", ".babelrc"], 'annotations-plugin');
                    spawn("npm", ["run", "build"], 'annotations-plugin');
                    console.log('mirador');
                    spawn("npm", ["run", "build:es"], 'mirador-video-annotation');
                    console.log('Workspaces compiled successfully');
                });
            }
        },
        new Dotenv()
    ]
};
